import React, { useState, memo } from "react";
import { FixedSizeList as List, areEqual } from "react-window";
import AutoSizer from "react-virtualized-auto-sizer";
import cls from "classnames";
import memoizeOne from "memoize-one";
import naturalCompare from "string-natural-compare";
import { FileTypeIcon } from "../file-type-icon/index";
import "./index.less";
var FILE_TYPES_ICON_MAP = {
  js: "js",
  jsx: "jsx",
  ts: "ts",
  tsx: "ts",
  json: "json",
  scss: "scss",
  css: "css",
  less: "less",
  tsconfig: "tsconfig",
  html: "html"
};
function getFileIcon(file) {
  var type = FILE_TYPES_ICON_MAP[(file || "").split(".").pop() || ""] || "text";
  return /*#__PURE__*/React.createElement(FileTypeIcon, {
    type: type
  });
}
var Row = /*#__PURE__*/memo(function (_ref) {
  var data = _ref.data,
    index = _ref.index,
    style = _ref.style;
  var flattenedData = data.flattenedData,
    onOpen = data.onOpen,
    onSelect = data.onSelect,
    selectedKeys = data.selectedKeys;
  var node = flattenedData[index];
  var left = node.depth * 20;
  return /*#__PURE__*/React.createElement("div", {
    className: cls("tree-node", {
      "tree-node-selected": selectedKeys === null || selectedKeys === void 0 ? void 0 : selectedKeys.includes(node.id)
    }),
    style: style,
    onClick: function onClick() {
      return onOpen(node);
    }
  }, /*#__PURE__*/React.createElement("div", {
    className: cls("tree-item", {
      "tree-folder": node.folder,
      "tree-item-expanded": node.expanded,
      "tree-item-open": !node.expanded
    }),
    onClick: function onClick(e) {
      return onSelect(e, node);
    },
    style: {
      left: left + "px",
      width: "calc(100% - " + left + "px)"
    }
  }, !node.folder && getFileIcon(node.label), /*#__PURE__*/React.createElement("span", {
    className: "label-name"
  }, node.label)));
}, areEqual);
var getItemData = memoizeOne(function (onOpen, onSelect, flattenedData, selectedKeys) {
  return {
    onOpen: onOpen,
    onSelect: onSelect,
    flattenedData: flattenedData,
    selectedKeys: selectedKeys
  };
});
export var FileTree = function FileTree(props) {
  var modules = props.modules,
    _props$selectedKeys = props.selectedKeys,
    selectedKeys = _props$selectedKeys === void 0 ? [] : _props$selectedKeys;
  var _useState = useState([]),
    openedNodeIds = _useState[0],
    setOpenedNodeIds = _useState[1];
  var onOpen = function onOpen(node) {
    if (!node.folder) return;
    !node.expanded ? setOpenedNodeIds([].concat(openedNodeIds, [node.id])) : setOpenedNodeIds(openedNodeIds.filter(function (id) {
      return id !== node.id;
    }));
  };
  var onSelect = function onSelect(e, node) {
    var _props$onSelect;
    //e.stopPropagation();
    if (node.folder) return;
    (_props$onSelect = props.onSelect) === null || _props$onSelect === void 0 ? void 0 : _props$onSelect.call(props, node.id);
  };
  var flattenedData = createFlattenTree(modules, openedNodeIds);
  var itemData = getItemData(onOpen, onSelect, flattenedData, selectedKeys);
  return /*#__PURE__*/React.createElement(AutoSizer, null, function (_ref2) {
    var height = _ref2.height,
      width = _ref2.width;
    return /*#__PURE__*/React.createElement(List, {
      className: "code-file-tree",
      height: height,
      itemCount: flattenedData.length,
      itemSize: 22,
      width: width,
      itemKey: function itemKey(index) {
        return flattenedData[index].id;
      },
      itemData: itemData
    }, Row);
  });
};
export function createFlattenTree(data, expandedKeys) {
  var flatTree = [];
  var filePaths = Object.keys(data);
  // console.log(JSON.parse(JSON.stringify(filePaths)))
  // filePaths.sort((a, b) => {
  //   // 检查字符串a是否包含斜杠
  //   const hasSlashA = a.substring(1).lastIndexOf('/');
  //   // 检查字符串b是否包含斜杠
  //   const hasSlashB = b.substring(1).lastIndexOf('/');
  //   console.log('---a', hasSlashA)
  //   console.log('---b', hasSlashB)
  //   // 如果a包含斜杠，而b不包含，则a应该排在b之后
  //   if (hasSlashA > -1 && hasSlashB > -1) {
  //     return hasSlashB - hasSlashA;
  //   }

  //   return naturalCompare(a, b, {
  //     caseInsensitive: true,
  //   }); // 包含斜杠的字符串按字典顺序排序
  // });
  // console.log(filePaths)
  filePaths.forEach(function (key) {
    var parts = key.split("/");
    var fileNames = [];
    var _loop = function _loop() {
      var path = parts.slice(0, i).join("/");
      var expanded = expandedKeys.includes(path);
      if (path && !flatTree.some(function (node) {
        return node.id === path;
      })) {
        fileNames.push({
          id: path,
          label: parts[i - 1] || "/",
          // 当前层的名称
          folder: i < parts.length,
          // 如果不是最后一层，则为文件夹
          depth: i - 1,
          // 层级深度，从 0 开始
          expanded: expanded // 默认展开
        });
      }
      if (expanded) return 1; // break
    };
    for (var i = 1; i <= parts.length; i++) {
      if (_loop()) break;
    }
    // console.log('--fileNames', fileNames);
    fileNames.sort(function (a, b) {
      return naturalCompare(a.id, b.id, {
        caseInsensitive: true
      });
    });
    flatTree.push.apply(flatTree, fileNames);
  });
  return flatTree;
}