import _Splitter from "antd/es/splitter";
import _extends from "@babel/runtime/helpers/extends";
import React, { useEffect, useRef, useState } from "react";
import Editor from "@gant-lowcode/lowcode-plugin-base-monaco-editor";
import "@gant-lowcode/lowcode-plugin-base-monaco-editor/lib/style.js";
import { FileTree } from "../file-tree/index";
import "./index.less";
var DEBOUNCE_UPDATE_INTERVAL_IN_MS = 500;
export function SourcesView(_ref) {
  var _state$currentFile, _state$currentFile2, _state$currentFile3, _state$currentFile4, _state$currentFile5;
  var code = _ref.code,
    onCodeChange = _ref.onCodeChange;
  var editorContainerRef = useRef(null);
  var _useState = useState(function () {
      var allFiles = Object.values(code.modules);
      var currentFile = [allFiles.find(function (x) {
        return /pages.+(js|ts)x?$/.test(x.fpath);
      }), allFiles.find(function (m) {
        return m.entry;
      })].filter(Boolean)[0];
      return {
        currentFile: currentFile,
        selectedKeys: [currentFile === null || currentFile === void 0 ? void 0 : currentFile.fpath]
      };
    }),
    state = _useState[0],
    setState = _useState[1];
  var _useState2 = useState(0),
    height = _useState2[0],
    setHeight = _useState2[1];
  useEffect(function () {
    var observer = new ResizeObserver(function () {
      if (editorContainerRef.current) {
        var _editorContainerRef$c = editorContainerRef.current.getBoundingClientRect(),
          _height = _editorContainerRef$c.height;
        setHeight(_height - 2);
      }
    });
    if (editorContainerRef.current) {
      observer.observe(editorContainerRef.current);
    }
    return function () {
      if (editorContainerRef.current) {
        observer.unobserve(editorContainerRef.current);
      }
    };
  }, []);
  var ref = useRef({
    debounceTimer: null
  });
  var onSelect = function onSelect(key) {
    setState(function (prev) {
      if (ref.current.debounceTimer) {
        ref.current.debounceTimer = null;
      }
      return _extends({}, prev, {
        currentFile: code.modules[key] || prev.currentFile
      });
    });
  };
  return /*#__PURE__*/React.createElement(_Splitter, {
    className: "sources-panes"
  }, /*#__PURE__*/React.createElement(_Splitter.Panel, {
    defaultSize: 300,
    min: 300,
    max: "70%",
    className: "file-tree-pane"
  }, /*#__PURE__*/React.createElement(FileTree, {
    modules: code.modules,
    onSelect: onSelect,
    selectedKeys: (_state$currentFile = state.currentFile) !== null && _state$currentFile !== void 0 && _state$currentFile.fpath ? [(_state$currentFile2 = state.currentFile) === null || _state$currentFile2 === void 0 ? void 0 : _state$currentFile2.fpath] : undefined
  })), /*#__PURE__*/React.createElement(_Splitter.Panel, {
    className: "source-code-pane"
  }, /*#__PURE__*/React.createElement("div", {
    ref: editorContainerRef,
    style: {
      height: "100%"
    }
  }, /*#__PURE__*/React.createElement(Editor, {
    height: height,
    language: getFileLanguage((_state$currentFile3 = state.currentFile) === null || _state$currentFile3 === void 0 ? void 0 : _state$currentFile3.fpath),
    saveViewState: true,
    defaultValue: (_state$currentFile4 = state.currentFile) === null || _state$currentFile4 === void 0 ? void 0 : _state$currentFile4.code,
    path: (_state$currentFile5 = state.currentFile) === null || _state$currentFile5 === void 0 ? void 0 : _state$currentFile5.fpath,
    onChange: function onChange(value) {
      console.debug("[monaco editor] onChange: %o (currentFile: %o)", {
        value: value
      }, state.currentFile);
      var currentFile = state.currentFile;
      if (currentFile) {
        if (ref.current.debounceTimer) {
          clearTimeout(ref.current.debounceTimer);
        }
        ref.current.debounceTimer = setTimeout(function () {
          var _extends2;
          ref.current.debounceTimer = null;
          onCodeChange(_extends({}, code, {
            modules: _extends({}, code.modules, (_extends2 = {}, _extends2[currentFile.fpath] = _extends({}, currentFile, {
              code: value || ""
            }), _extends2))
          }));
        }, DEBOUNCE_UPDATE_INTERVAL_IN_MS);
      }
    }
  }))));
}
function getFileLanguage(file) {
  switch ((file || "").split(".").pop()) {
    case "ts":
    case "tsx":
      return "typescript";
    case "js":
    case "jsx":
      return "javascript";
    case "css":
      return "css";
    case "scss":
      return "scss";
    case "less":
      return "less";
    case "json":
      return "json";
    case "md":
      return "markdown";
    default:
      return "text";
  }
}