"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
exports.__esModule = true;
exports.FileTree = void 0;
exports.createFlattenTree = createFlattenTree;
var _react = _interopRequireWildcard(require("react"));
var _reactWindow = require("react-window");
var _reactVirtualizedAutoSizer = _interopRequireDefault(require("react-virtualized-auto-sizer"));
var _classnames = _interopRequireDefault(require("classnames"));
var _memoizeOne = _interopRequireDefault(require("memoize-one"));
var _stringNaturalCompare = _interopRequireDefault(require("string-natural-compare"));
var _index = require("../file-type-icon/index");
require("./index.less");
function _getRequireWildcardCache(e) { if ("function" != typeof WeakMap) return null; var r = new WeakMap(), t = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(e) { return e ? t : r; })(e); }
function _interopRequireWildcard(e, r) { if (!r && e && e.__esModule) return e; if (null === e || "object" != typeof e && "function" != typeof e) return { "default": e }; var t = _getRequireWildcardCache(r); if (t && t.has(e)) return t.get(e); var n = { __proto__: null }, a = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var u in e) if ("default" !== u && {}.hasOwnProperty.call(e, u)) { var i = a ? Object.getOwnPropertyDescriptor(e, u) : null; i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u]; } return n["default"] = e, t && t.set(e, n), n; }
var FILE_TYPES_ICON_MAP = {
  js: "js",
  jsx: "jsx",
  ts: "ts",
  tsx: "ts",
  json: "json",
  scss: "scss",
  css: "css",
  less: "less",
  tsconfig: "tsconfig",
  html: "html"
};
function getFileIcon(file) {
  var type = FILE_TYPES_ICON_MAP[(file || "").split(".").pop() || ""] || "text";
  return /*#__PURE__*/_react["default"].createElement(_index.FileTypeIcon, {
    type: type
  });
}
var Row = /*#__PURE__*/(0, _react.memo)(function (_ref) {
  var data = _ref.data,
    index = _ref.index,
    style = _ref.style;
  var flattenedData = data.flattenedData,
    onOpen = data.onOpen,
    onSelect = data.onSelect,
    selectedKeys = data.selectedKeys;
  var node = flattenedData[index];
  var left = node.depth * 20;
  return /*#__PURE__*/_react["default"].createElement("div", {
    className: (0, _classnames["default"])("tree-node", {
      "tree-node-selected": selectedKeys === null || selectedKeys === void 0 ? void 0 : selectedKeys.includes(node.id)
    }),
    style: style,
    onClick: function onClick() {
      return onOpen(node);
    }
  }, /*#__PURE__*/_react["default"].createElement("div", {
    className: (0, _classnames["default"])("tree-item", {
      "tree-folder": node.folder,
      "tree-item-expanded": node.expanded,
      "tree-item-open": !node.expanded
    }),
    onClick: function onClick(e) {
      return onSelect(e, node);
    },
    style: {
      left: left + "px",
      width: "calc(100% - " + left + "px)"
    }
  }, !node.folder && getFileIcon(node.label), /*#__PURE__*/_react["default"].createElement("span", {
    className: "label-name"
  }, node.label)));
}, _reactWindow.areEqual);
var getItemData = (0, _memoizeOne["default"])(function (onOpen, onSelect, flattenedData, selectedKeys) {
  return {
    onOpen: onOpen,
    onSelect: onSelect,
    flattenedData: flattenedData,
    selectedKeys: selectedKeys
  };
});
var FileTree = exports.FileTree = function FileTree(props) {
  var modules = props.modules,
    _props$selectedKeys = props.selectedKeys,
    selectedKeys = _props$selectedKeys === void 0 ? [] : _props$selectedKeys;
  var _useState = (0, _react.useState)([]),
    openedNodeIds = _useState[0],
    setOpenedNodeIds = _useState[1];
  var onOpen = function onOpen(node) {
    if (!node.folder) return;
    !node.expanded ? setOpenedNodeIds([].concat(openedNodeIds, [node.id])) : setOpenedNodeIds(openedNodeIds.filter(function (id) {
      return id !== node.id;
    }));
  };
  var onSelect = function onSelect(e, node) {
    var _props$onSelect;
    //e.stopPropagation();
    if (node.folder) return;
    (_props$onSelect = props.onSelect) === null || _props$onSelect === void 0 ? void 0 : _props$onSelect.call(props, node.id);
  };
  var flattenedData = createFlattenTree(modules, openedNodeIds);
  var itemData = getItemData(onOpen, onSelect, flattenedData, selectedKeys);
  return /*#__PURE__*/_react["default"].createElement(_reactVirtualizedAutoSizer["default"], null, function (_ref2) {
    var height = _ref2.height,
      width = _ref2.width;
    return /*#__PURE__*/_react["default"].createElement(_reactWindow.FixedSizeList, {
      className: "code-file-tree",
      height: height,
      itemCount: flattenedData.length,
      itemSize: 22,
      width: width,
      itemKey: function itemKey(index) {
        return flattenedData[index].id;
      },
      itemData: itemData
    }, Row);
  });
};
function createFlattenTree(data, expandedKeys) {
  var flatTree = [];
  var filePaths = Object.keys(data);
  // console.log(JSON.parse(JSON.stringify(filePaths)))
  // filePaths.sort((a, b) => {
  //   // 检查字符串a是否包含斜杠
  //   const hasSlashA = a.substring(1).lastIndexOf('/');
  //   // 检查字符串b是否包含斜杠
  //   const hasSlashB = b.substring(1).lastIndexOf('/');
  //   console.log('---a', hasSlashA)
  //   console.log('---b', hasSlashB)
  //   // 如果a包含斜杠，而b不包含，则a应该排在b之后
  //   if (hasSlashA > -1 && hasSlashB > -1) {
  //     return hasSlashB - hasSlashA;
  //   }

  //   return naturalCompare(a, b, {
  //     caseInsensitive: true,
  //   }); // 包含斜杠的字符串按字典顺序排序
  // });
  // console.log(filePaths)
  filePaths.forEach(function (key) {
    var parts = key.split("/");
    var fileNames = [];
    var _loop = function _loop() {
      var path = parts.slice(0, i).join("/");
      var expanded = expandedKeys.includes(path);
      if (path && !flatTree.some(function (node) {
        return node.id === path;
      })) {
        fileNames.push({
          id: path,
          label: parts[i - 1] || "/",
          // 当前层的名称
          folder: i < parts.length,
          // 如果不是最后一层，则为文件夹
          depth: i - 1,
          // 层级深度，从 0 开始
          expanded: expanded // 默认展开
        });
      }
      if (expanded) return 1; // break
    };
    for (var i = 1; i <= parts.length; i++) {
      if (_loop()) break;
    }
    // console.log('--fileNames', fileNames);
    fileNames.sort(function (a, b) {
      return (0, _stringNaturalCompare["default"])(a.id, b.id, {
        caseInsensitive: true
      });
    });
    flatTree.push.apply(flatTree, fileNames);
  });
  return flatTree;
}